/*
  serial.c - Low level functions for sending and recieving bytes via the serial port
  Part of Grbl

  Copyright (c) 2009-2011 Simen Svale Skogsrud
  Copyright (c) 2011-2012 Sungeun K. Jeon

  Grbl is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Grbl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Grbl.  If not, see <http://www.gnu.org/licenses/>.
*/

/* This code was initially inspired by the wiring_serial module by David A. Mellis which
   used to be a part of the Arduino project. */ 

#include "TwinTeeth.h"
#include "MySerial.h"

char rx_buffer[RX_BUFFER_SIZE];
unsigned char rx_buffer_head = 0;
unsigned char rx_buffer_tail = 0;

// Constructors ////////////////////////////////////////////////////////////////

MySerial::MySerial()
{
	overflow = false;
	rts_flag = false;
}

// Public Methods //////////////////////////////////////////////////////////////

void MySerial::begin(long baudrate)
{
	uint16_t UBRR0_value;

  // Set baud rate
  if (baudrate < 57600){
    UBRR0_value = ((F_CPU / (8L * baudrate)) - 1)/2 ;
    UCSR0A &= ~(1 << U2X0); // baud doubler off  - Only needed on Uno XXX
  }
  else {
    UBRR0_value = ((F_CPU / (4L * baudrate)) - 1)/2;
    UCSR0A |= (1 << U2X0);  // baud doubler on for high baud rates, i.e. 115200
  }

  UBRR0H = UBRR0_value >> 8;
  UBRR0L = UBRR0_value;
            
  // enable rx and tx
  UCSR0B |= 1<<RXEN0;
  UCSR0B |= 1<<TXEN0;
  UCSR0B |= 1<<RXCIE0;

  // defaults to 8-bit, no parity, 1 stop bit
}

void MySerial::end()
{
  cbi(UCSR0B, RXEN0);
  cbi(UCSR0B, TXEN0);
  cbi(UCSR0B, RXCIE0);  
}


int MySerial::peek(void)
{
  if (rx_buffer_head == rx_buffer_tail) {
    return SERIAL_NO_DATA;
  } else {
    return rx_buffer[rx_buffer_tail];
  }
}

char MySerial::read()
{
	if (rx_buffer_head == rx_buffer_tail) {
		if (rts_flag) send_cts();
		return SERIAL_NO_DATA;
	} else {
		char data = rx_buffer[rx_buffer_tail];
		rx_buffer_tail++;
		if (rx_buffer_tail == RX_BUFFER_SIZE) { rx_buffer_tail = 0; }
		if (rts_flag && (free_room() > PTRL_BLOCK_SIZE)) { 
			send_cts();
		}
		return data;
	}    
}

void MySerial::flush(){
  rx_buffer_tail = rx_buffer_head;
  send_cts();
}


/// imports from print.h


void MySerial::print(char c, int base)
{
  print((long) c, base);
}

void MySerial::print(unsigned char b, int base)
{
  print((unsigned long) b, base);
}

void MySerial::print(int n, int base)
{
  print((long) n, base);
}

void MySerial::print(unsigned int n, int base)
{
  print((unsigned long) n, base);
}

void MySerial::print(long n, int base)
{
  if (base == 0) {
    write(n);
  } else if (base == 10) {
    if (n < 0) {
      print('-');
      n = -n;
    }
    printNumber(n, 10);
  } else {
    printNumber(n, base);
  }
}

void MySerial::print(unsigned long n, int base)
{
  if (base == 0) write(n);
  else printNumber(n, base);
}

void MySerial::print(double n, int digits)
{
  printFloat(n, digits);
}

void MySerial::println(void)
{
  print('\r');
  print('\n');  
}

void MySerial::println(const String &s)
{
  print(s);
  println();
}

void MySerial::println(const char c[])
{
  print(c);
  println();
}

void MySerial::println(char c, int base)
{
  print(c, base);
  println();
}

void MySerial::println(unsigned char b, int base)
{
  print(b, base);
  println();
}

void MySerial::println(int n, int base)
{
  print(n, base);
  println();
}

void MySerial::println(unsigned int n, int base)
{
  print(n, base);
  println();
}

void MySerial::println(long n, int base)
{
  print(n, base);
  println();
}

void MySerial::println(unsigned long n, int base)
{
  print(n, base);
  println();
}

void MySerial::println(double n, int digits)
{
  print(n, digits);
  println();
}

// Private Methods /////////////////////////////////////////////////////////////

void MySerial::printNumber(unsigned long n, uint8_t base)
{
  unsigned char buf[8 * sizeof(long)]; // Assumes 8-bit chars. 
  unsigned long i = 0;

  if (n == 0) {
    print('0');
    return;
  } 

  while (n > 0) {
    buf[i++] = n % base;
    n /= base;
  }

  for (; i > 0; i--)
    print((char) (buf[i - 1] < 10 ?
      '0' + buf[i - 1] :
      'A' + buf[i - 1] - 10));
}

void MySerial::printBinary(byte n)
{
    int z;

    for (z = 128; z > 0; z >>= 1)
    {
        print(((n & z) == z) ? '1' : '0');
    }
}

void MySerial::printFloat(double number, uint8_t digits) 
{ 
  // Handle negative numbers
  if (number < 0.0)
  {
     print('-');
     number = -number;
  }

  // Round correctly so that print(1.999, 2) prints as "2.00"
  double rounding = 0.5;
  for (uint8_t i=0; i<digits; ++i)
    rounding /= 10.0;
  
  number += rounding;

  // Extract the integer part of the number and print it
  unsigned long int_part = (unsigned long)number;
  double remainder = number - (double)int_part;
  print(int_part);

  // Print the decimal point, but only if there are digits beyond
  if (digits > 0)
    print("."); 

  // Extract digits from the remainder one at a time
  while (digits-- > 0)
  {
    remainder *= 10.0;
    int toPrint = int(remainder);
    print(toPrint);
    remainder -= toPrint; 
  } 
}

// Pre-instantiate objet

MySerial MSerial;


// IRQ vector
// It mainly works when motors are stopped
//
ISR(USART0_RX_vect)
{
	MSerial.checkRx();
}


